# coding: utf8
from future.builtins import object
import abc

from sinops import exceptions
from future.utils import with_metaclass


class VerifierBase(with_metaclass(abc.ABCMeta, object)):

    u"""
    Базовый класс для проверки ЭЦП.

    Наследники должны реализовывать метод verify
    """

    @classmethod
    def is_suitable(cls, signature):
        u"""
        Должен определять может ли данный бэкенд верифицировать
        подпись `signature`.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def verify(self, signature):
        u"""
        Метод должен проверять ЭЦП сообщения.

        В случае успешного выполнения должен возвращать None, иначе
        выбрасывать исключение VerificationError.
        """

        if not self.is_suitable(signature):
            raise exceptions.UnsuitableSignatureType
